/*
	MIT License http://www.opensource.org/licenses/mit-license.php
	Author Tobias Koppers @sokra
*/
var path = require("path");
var parseSource = require("./parseSource");
var ReplaceMany = require("./ReplaceMany");
var loaderUtils = require("loader-utils");
var SourceListMap = require("source-list-map").SourceListMap;
var CleanCSS = require("clean-css");
var getLocalIdent = require("./getLocalIdent");

module.exports = function(content, map) {
	if(this.cacheable) this.cacheable();
	var query = loaderUtils.parseQuery(this.query);
	var root = query.root;
	var forceMinimize = query.minimize;
	var importLoaders = parseInt(query.importLoaders, 10) || 0;
	var minimize = typeof forceMinimize !== "undefined" ? !!forceMinimize : (this && this.minimize);
	var localIdentName = query.localIdentName || "[hash:base64]";
	var localIdentRegExp = query.localIdentRegExp;

	if(typeof map !== "string") {
		map = JSON.stringify(map);
	}

	var result = [];

	// for importing CSS
	var loadersRequest = this.loaders.slice(
		this.loaderIndex,
		this.loaderIndex + 1 + importLoaders
	).map(function(x) { return x.request; }).join("!");
	var importUrlPrefix = "-!" + loadersRequest + "!";

	var stuff = parseSource(content);

	var replacer = new ReplaceMany();

	// store already imported files
	var importedUrls = [];

	// add @imports to result
	stuff.imports.forEach(function(imp) {
		replacer.replace(imp.start, imp.length, "");
		if(!loaderUtils.isUrlRequest(imp.url, root)) {
			result.push("exports.push([module.id, " +
				JSON.stringify("@import url(" + imp.url + ");") + ", " +
				JSON.stringify(imp.mediaQuery) + "]);");
		} else {
			var importUrl = importUrlPrefix + loaderUtils.urlToRequest(imp.url);
			result.push("exports.i(require(" + loaderUtils.stringifyRequest(this, importUrl) + "), " + JSON.stringify(imp.mediaQuery) + ");");
			if(!imp.mediaQuery)
				importedUrls.push(importUrl);
		}
	}, this);

	// replace url(...)
	if(query.url !== false) {
		stuff.urls.forEach(function(url, idx) {
			replacer.replace(url.start, url.length, "__CSSLOADERURL_" + idx + "__");
		});
	}

	// replace :local()
	var locals = {};
	var localExtends = {};
	stuff.locals.forEach(function(local) {
		var ident;
		var name = local.name;
		if(!locals[name]) {
			ident = getLocalIdent(this, localIdentName, name, {
				regExp: localIdentRegExp
			});
			locals[name] = ident;
		} else {
			ident = locals[name];
		}
		if(local.extends) {
			local.extends.forEach(function(extend) {
				if(!localExtends[name])
					localExtends[name] = [];
				localExtends[name].push(extend);
			});
		}
		replacer.replace(local.start, local.length, local.prefix + ident);
	}, this);

	// remove stuff
	stuff.remove.forEach(function(rem) {
		replacer.replace(rem.start, rem.length, "");
	});

	// pass errors from parser
	if(this.emitError) {
		stuff.errors.forEach(function(err) {
			this.emitError(err);
		}, this);
	}

	// generate the locals
	var localKeys = Object.keys(locals);
	if(localKeys.length > 0) {
		var localLines = localKeys.map(function(key, idx) {
			var line = "  " + JSON.stringify(key) + ": ";
			function addExtend(extend) {
				if(extend.from) {
					var importUrl = importUrlPrefix + extend.from;
					if(importedUrls.indexOf(importUrl) < 0) {
						result.push("exports.i(require(" + loaderUtils.stringifyRequest(this, importUrl) + "), \"\");");
						importedUrls.push(importUrl);
					}
					line += " + \" \" + require(" + loaderUtils.stringifyRequest(this, importUrl) + ").locals[" + JSON.stringify(extend.name) + "]";
				} else if(locals[extend.name]) {
					line += " + \" \" + " + JSON.stringify(locals[extend.name]);
					if(localExtends[extend.name]) {
						localExtends[extend.name].forEach(addExtend, this);
					}
				} else if(this.emitError) {
					this.emitError("Cannot extend from unknown class '" + extend.name + "'");
				}
			}
			line += JSON.stringify(locals[key]);
			if(localExtends[key]) {
				localExtends[key].forEach(addExtend, this);
			}
			if(idx !== localKeys.length - 1) line += ",";
			return line;
		}, this);
		result.push("exports.locals = {");
		result.push(localLines.join("\n"));
		result.push("};");
	}

	// transform the CSS
	var cssContent = replacer.run(content);

	// minimize CSS
	if(minimize) {
		var options = Object.create(query);
		if(query.sourceMap && map) {
			options.sourceMap = map;
		}
		var minimizeResult = new CleanCSS(options).minify(cssContent);
		map = minimizeResult.sourceMap;
		cssContent = minimizeResult.styles;
		if(typeof map !== "string")
			map = JSON.stringify(map);
	}

	function toEmbStr(str) {
		return JSON.stringify(str).replace(/^"|"$/g, "");
	}

	// replace url(...) in the generated code
	var css = JSON.stringify(cssContent);
	var urlRegExp = /__CSSLOADERURL_[0-9]+__/g;
	css = css.replace(urlRegExp, function(str) {
		var match = /^__CSSLOADERURL_([0-9]+)__$/.exec(str);
		if(!match) return str;
		var idx = parseInt(match[1], 10);
		if(!stuff.urls[idx]) return str;
		var urlItem = stuff.urls[idx];
		var url = urlItem.url;
		if(!loaderUtils.isUrlRequest(url, root))
			return toEmbStr(urlItem.raw);
		idx = url.indexOf("?#");
		if(idx < 0) idx = url.indexOf("#");
		if(idx > 0) {
			// in cases like url('webfont.eot?#iefix')
			var urlRequest = url.substr(0, idx);
			return "\"+require(" + loaderUtils.stringifyRequest(this, loaderUtils.urlToRequest(urlRequest, root)) + ")+\"" + url.substr(idx);
		} else if(idx === 0) {
			// only hash
			return toEmbStr(urlItem.raw);
		}
		return "\"+require(" + loaderUtils.stringifyRequest(this, loaderUtils.urlToRequest(url, root)) + ")+\"";
	}.bind(this));

	// add a SourceMap
	if(query.sourceMap && !minimize) {
		var cssRequest = loaderUtils.getRemainingRequest(this);
		var request = loaderUtils.getCurrentRequest(this);
		if(!map) {
			var sourceMap = new SourceListMap();
			sourceMap.add(content, cssRequest, content);
			map = sourceMap.toStringWithSourceMap({
				file: request
			}).map;
			if(map.sources) {
				map.sources = map.sources.map(function(source) {
					var p = path.relative(query.context || this.options.context, source).replace(/\\/g, "/");
					if(p.indexOf("../") !== 0)
						p = "./" + p;
					return "/" + p;
				}, this);
				map.sourceRoot = "webpack://";
			}
			map = JSON.stringify(map);
		}
		result.push("exports.push([module.id, " + css + ", \"\", " + map + "]);");
	} else {
		result.push("exports.push([module.id, " + css + ", \"\"]);");
	}

	// embed runtime
	return "exports = module.exports = require(" + loaderUtils.stringifyRequest(this, require.resolve("./css-base.js")) + ")();\n" +
		result.join("\n");
};
